Physic = Class{}

function Physic:__init(x, y, w, h)
	self.x        = x
	self.y        = y
	self.w        = w
	self.h        = h
	self.xspeed   = 0
	self.yspeed   = 0
	self.onground = false

	Bump.add(self)
end

function Physic:update(dt)
	--gravity
	if not self.onground then
		self.yspeed = self.yspeed + gameplay.gravity * dt
	end

	--limit speed
	if self.yspeed > 1500 then
		self.yspeed = 1500
	end

	--final movement
	self.x = self.x + self.xspeed * dt
	self.y = self.y + self.yspeed * dt

	--set onground to false for next frame
	self.onground = false
end

function Physic:collision(item, dx, dy)
	if item.name == 'block' then
		--landing
		if dy < 0 then
			self.y = self.y + dy
			self.onground = true
			self.yspeed = 0
		end
		--hit head on ceiling
		if dy > 0 then
			self.y = self.y + dy
			self.yspeed = 0
		end
		--side collisions
		if not (dx == 0) then
			self.x = self.x + dx
			self.xspeed = 0
		end
	end
end



--player
Player = Physic:extends{name = 'player'}

function Player:__init(x, y)
	self.super.__init(self, x, y, gameplay.block_size, gameplay.block_size)
	self.spawn = {x = self.x, y = self.y}
end

function Player:update(dt)
	--jumping
	if love.keyboard.isDown('up') and self.onground then
		self.onground = false
		self.yspeed = -750 - 125 * (math.abs(self.xspeed) / 750)
	end

	--variable jumping
	if not love.keyboard.isDown('up') and self.yspeed < 0 then
		self.yspeed = self.yspeed - self.yspeed * 10 * dt
	end

	--running
	if love.keyboard.isDown('left') then
		self.xspeed = self.xspeed - 1200 * dt
	elseif love.keyboard.isDown('right') then
		self.xspeed = self.xspeed + 1200 * dt
	else
		if self.xspeed > 0 then
			self.xspeed = self.xspeed - 1200 * dt
			if self.xspeed < 0 then
				self.xspeed = 0
			end
		end
		if self.xspeed < 0 then
			self.xspeed = self.xspeed + 1200 * dt
			if self.xspeed > 0 then
				self.xspeed = 0
			end
		end
	end

	--limit horizontal speed
	if self.xspeed < -750 then
		self.xspeed = -750
	elseif self.xspeed > 750 then
		self.xspeed = 750
	end

	self.super.update(self, dt)

	--update camera
	gameplay.camera.x = self.x - 960
	gameplay.camera.y = self.y - 540
	if gameplay.camera.x > gameplay.level_width - 1920 then
		gameplay.camera.x = gameplay.level_width - 1920
	end
	if gameplay.camera.y > gameplay.level_height - 1080 then
		gameplay.camera.y = gameplay.level_height - 1080
	end
end

function Player:collision(item, dx, dy)
	self.super.collision(self, item, dx, dy)

	if item.name == 'hazard' or item.name == 'enemy' then
		self:die()
	end
	if item.name == 'spawn' then
		self.spawn.x = item.x
		self.spawn.y = item.y
	end
end

function Player:die()
	--teleport to last spawn point
	self.x = self.spawn.x
	self.y = self.spawn.y
	self.xspeed = 0
	self.yspeed = 0
end

function Player:draw(x, y)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.rectangle('fill', self.x + x, self.y + y, self.w, self.h)
end



--normal enemy
Enemy = Physic:extends{name = 'enemy'}

function Enemy:__init(x, y)
	self.super.__init(self, x, y, gameplay.block_size, gameplay.block_size)

	--start out moving left
	self.xspeed = -150
end

function Enemy:collision(item, dx, dy)
	self.super.collision(self, item, dx, dy)

	--reverse when hitting walls
	if item.name == 'block' and not (dx == 0) then
		if dx > 0 then
			self.xspeed = 150
		else
			self.xspeed = -150
		end
	end
end

function Enemy:draw(x, y)
	love.graphics.setColor(255, 106, 0, 255)
	love.graphics.rectangle('fill', self.x + x, self.y + y, self.w, self.h)
end



--fancy enemy
FancyEnemy = Physic:extends{name = 'fancy enemy'}

function FancyEnemy:__init(x, y)
	self.super.__init(self, x, y, gameplay.block_size, gameplay.block_size)

	--periodic jumping
	self.timer = Timer.new()
	self.timer:addPeriodic(4 + 2 * math.random(), function()
		self.yspeed = -400
		self.onground = false
	end)

	--start out moving left
	self.xspeed = -150
end

function FancyEnemy:update(dt)
	self.timer:update(dt)
end

function FancyEnemy:collision(item, dx, dy)
	self.super.collision(self, item, dx, dy)

	--reverse when hitting walls
	if item.name == 'block' and not (dx == 0) then
		if dx > 0 then
			self.xspeed = 150
		else
			self.xspeed = -150
		end
	end
end

function FancyEnemy:draw(x, y)
	love.graphics.setColor(9, 100, 0, 255)
	love.graphics.rectangle('fill', self.x + x, self.y + y, self.w, self.h)
end