gameplay = Gamestate.new()

--collision callbacks
function Bump.getBBox(item)
	return item.x, item.y, item.w, item.h
end

function Bump.collision(item1, item2, dx, dy)
	if item1.collision then
		item1:collision(item2, dx, dy)
	end
	if item2.collision then
		item2:collision(item1, -dx, -dy)
	end
end

function gameplay:enter()
	--constants
	self.block_size = 40
	self.gravity = 1700

	--camera
	self.camera = {x = 0, y = 0}

	--set up instances
	self.instance = {}
	table.insert(self.instance, Master())

	self.canvas = love.graphics.newCanvas(1920, 1080)
end

function gameplay:update(dt)
	--dont update if fps is too low
	if dt > .03 then
		return false
	end

	--update all instances
	for i = 1, #self.instance do
		if self.instance[i].update then
			self.instance[i]:update(dt)
		end
	end

	Bump.collide()
end

function gameplay:draw()
	local i

	self.canvas:renderTo(function()
		--draw black bg to avoid smearing
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.rectangle('fill', 0, 0, 1920, 1080)

		--draw all instances
		for i = 1, #self.instance do
			if self.instance[i].draw then
				if self.instance[i].x > self.camera.x - 100 and self.instance[i].x < self.camera.x + 2020 and self.instance[i].y > self.camera.y - 100 and self.instance[i].y < self.camera.y + 1180 then
					self.instance[i]:draw(-self.camera.x, -self.camera.y)
				end
			end
		end
	end)

	--draw graphics scaled
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(self.canvas, 0, 0, 0, love.graphics.getHeight() / 1080, love.graphics.getHeight() / 1080)

	love.graphics.print(gameplay.level_width..' '..gameplay.level_height, 0, 0)
end